@@ -1,9 +1,9 @@
 """Sub-module providing 'keyboard awareness'."""
 
 # std imports
-import platform
-import time
 import re
+import time
+import platform
 
 # 3rd party
 import six
@@ -12,10 +12,7 @@ try:
     from collections import OrderedDict
 except ImportError:
     # python 2.6 requires 3rd party library (backport)
-    #
-    # pylint: disable=import-error
-    #         Unable to import 'ordereddict'
-    from ordereddict import OrderedDict
+    from ordereddict import OrderedDict  # pylint: disable=import-error
 
 # curses
 if platform.system() == 'Windows':
@@ -80,10 +77,9 @@ def get_curses_keycodes():
     Return mapping of curses key-names paired by their keycode integer value.
 
     :rtype: dict
-
-    Returns dictionary of (name, code) pairs for curses keyboard constant
-    values and their mnemonic name. Such as code ``260``, with the value of
-    its key-name identity, ``u'KEY_LEFT'``.
+    :returns: Dictionary of (name, code) pairs for curses keyboard constant
+        values and their mnemonic name. Such as code ``260``, with the value of
+        its key-name identity, ``u'KEY_LEFT'``.
     """
     _keynames = [attr for attr in dir(curses)
                  if attr.startswith('KEY_')]
@@ -96,11 +92,12 @@ def get_keyboard_codes():
     Return mapping of keycode integer values paired by their curses key-name.
 
     :rtype: dict
+    :returns: Dictionary of (code, name) pairs for curses keyboard constant
+        values and their mnemonic name. Such as key ``260``, with the value of
+        its identity, ``u'KEY_LEFT'``.
 
-    Returns dictionary of (code, name) pairs for curses keyboard constant
-    values and their mnemonic name. Such as key ``260``, with the value of
-    its identity, ``u'KEY_LEFT'``.  These are derived from the attributes by
-    the same of the curses module, with the following exceptions:
+    These keys are derived from the attributes by the same of the curses module,
+    with the following exceptions:
 
     * ``KEY_DELETE`` in place of ``KEY_DC``
     * ``KEY_INSERT`` in place of ``KEY_IC``
@@ -118,6 +115,8 @@ def get_keyboard_codes():
     """
     keycodes = OrderedDict(get_curses_keycodes())
     keycodes.update(CURSES_KEYCODE_OVERRIDE_MIXIN)
+    # merge _CURSES_KEYCODE_ADDINS added to our module space
+    keycodes.update((name, value) for name, value in globals().items() if name.startswith('KEY_'))
 
     # invert dictionary (key, values) => (values, key), preferring the
     # last-most inserted value ('KEY_DELETE' over 'KEY_DC').
@@ -130,16 +129,17 @@ def _alternative_left_right(term):
 
     :arg blessed.Terminal term: :class:`~.Terminal` instance.
     :rtype: dict
+    :returns: Dictionary of sequences ``term._cuf1``, and ``term._cub1``,
+        valued as ``KEY_RIGHT``, ``KEY_LEFT`` (when appropriate).
 
     This function supports :func:`get_terminal_sequences` to discover
     the preferred input sequence for the left and right application keys.
 
-    Return dict of sequences ``term._cuf1``, and ``term._cub1``,
-    valued as ``KEY_RIGHT``, ``KEY_LEFT`` (when appropriate).  It is
-    necessary to check the value of these sequences to ensure we do not
+    It is necessary to check the value of these sequences to ensure we do not
     use ``u' '`` and ``u'\b'`` for ``KEY_RIGHT`` and ``KEY_LEFT``,
     preferring their true application key sequence, instead.
     """
+    # pylint: disable=protected-access
     keymap = dict()
     if term._cuf1 and term._cuf1 != u' ':
         keymap[term._cuf1] = curses.KEY_RIGHT
@@ -203,6 +203,7 @@ def get_leading_prefixes(sequences):
 
     :arg iterable sequences
     :rtype: set
+    :return: Set of all string prefixes
 
     Given an iterable of strings, all textparts leading up to the final
     string is returned as a unique set.  This function supports the
@@ -217,19 +218,20 @@ def get_leading_prefixes(sequences):
 
 def resolve_sequence(text, mapper, codes):
     r"""
-    Return :class:`Keystroke` instance for given sequence ``text``.
+    Return a single :class:`Keystroke` instance for given sequence ``text``.
 
-    The given ``text`` may extend beyond a matching sequence, such as
-    ``u\x1b[Dxxx`` returns a :class:`Keystroke` instance of attribute
-    :attr:`Keystroke.sequence` valued only ``u\x1b[D``.  It is up to
-    determine that ``xxx`` remains unresolved.
-
-    :arg text: string of characters received from terminal input stream.
+    :arg str text: string of characters received from terminal input stream.
     :arg OrderedDict mapper: unicode multibyte sequences, such as ``u'\x1b[D'``
         paired by their integer value (260)
     :arg dict codes: a :type:`dict` of integer values (such as 260) paired
         by their mnemonic name, such as ``'KEY_LEFT'``.
     :rtype: Keystroke
+    :returns: Keystroke instance for the given sequence
+
+    The given ``text`` may extend beyond a matching sequence, such as
+    ``u\x1b[Dxxx`` returns a :class:`Keystroke` instance of attribute
+    :attr:`Keystroke.sequence` valued only ``u\x1b[D``.  It is up to
+    calls to determine that ``xxx`` remains unresolved.
     """
     for sequence, code in mapper.items():
         if text.startswith(sequence):
@@ -286,68 +288,62 @@ def _read_until(term, pattern, timeout):
     # concerned about OOM conditions: only (human) keyboard input and terminal
     # response sequences are expected.
 
-    while True:
+    while True:  # pragma: no branch
         # block as long as necessary to ensure at least one character is
         # received on input or remaining timeout has elapsed.
         ucs = term.inkey(timeout=_time_left(stime, timeout))
-        if ucs:
+        # while the keyboard buffer is "hot" (has input), we continue to
+        # aggregate all awaiting data.  We do this to ensure slow I/O
+        # calls do not unnecessarily give up within the first 'while' loop
+        # for short timeout periods.
+        while ucs:
             buf += ucs
-            # while the keyboard buffer is "hot" (has input), we continue to
-            # aggregate all awaiting data.  We do this to ensure slow I/O
-            # calls do not unnecessarily give up within the first 'while' loop
-            # for short timeout periods.
-            while True:
-                ucs = term.inkey(timeout=0)
-                if not ucs:
-                    break
-                buf += ucs
+            ucs = term.inkey(timeout=0)
 
         match = re.search(pattern=pattern, string=buf)
         if match is not None:
             # match
             break
 
-        if timeout is not None:
-            if not _time_left(stime, timeout):
-                # timeout
-                break
+        if timeout is not None and not _time_left(stime, timeout):
+            # timeout
+            break
 
     return match, buf
 
 
-def _inject_curses_keynames():
-    r"""
-    Inject KEY_NAMES that we think would be useful into the curses module.
+#: Though we may determine *keynames* and codes for keyboard input that
+#: generate multibyte sequences, it is also especially useful to aliases
+#: a few basic ASCII characters such as ``KEY_TAB`` instead of ``u'\t'`` for
+#: uniformity.
+#:
+#: Furthermore, many key-names for application keys enabled only by context
+#: manager :meth:`~.Terminal.keypad` are surprisingly absent.  We inject them
+#: here directly into the curses module.
+_CURSES_KEYCODE_ADDINS = (
+    'TAB',
+    'KP_MULTIPLY',
+    'KP_ADD',
+    'KP_SEPARATOR',
+    'KP_SUBTRACT',
+    'KP_DECIMAL',
+    'KP_DIVIDE',
+    'KP_EQUAL',
+    'KP_0',
+    'KP_1',
+    'KP_2',
+    'KP_3',
+    'KP_4',
+    'KP_5',
+    'KP_6',
+    'KP_7',
+    'KP_8',
+    'KP_9')
 
-    This function compliments the global constant
-    :obj:`DEFAULT_SEQUENCE_MIXIN`.  It is important to note that this
-    function has the side-effect of **injecting** new attributes to the
-    curses module, and is called from the global namespace at time of
-    import.
-
-    Though we may determine *keynames* and codes for keyboard input that
-    generate multibyte sequences, it is also especially useful to aliases
-    a few basic ASCII characters such as ``KEY_TAB`` instead of ``u'\t'`` for
-    uniformity.
-
-    Furthermore, many key-names for application keys enabled only by context
-    manager :meth:`~.Terminal.keypad` are surprisingly absent.  We inject them
-    here directly into the curses module.
-
-    It is not necessary to directly "monkeypatch" the curses module to
-    contain these constants, as they will also be accessible as attributes
-    of the Terminal class instance, they are provided only for convenience
-    when mixed in with other curses code.
-    """
-    _lastval = max(get_curses_keycodes().values())
-    for key in ('TAB', 'KP_MULTIPLY', 'KP_ADD', 'KP_SEPARATOR', 'KP_SUBTRACT',
-                'KP_DECIMAL', 'KP_DIVIDE', 'KP_EQUAL', 'KP_0', 'KP_1', 'KP_2',
-                'KP_3', 'KP_4', 'KP_5', 'KP_6', 'KP_7', 'KP_8', 'KP_9'):
-        _lastval += 1
-        setattr(curses, 'KEY_{0}'.format(key), _lastval)
-
-
-_inject_curses_keynames()
+_LASTVAL = max(get_curses_keycodes().values())
+for keycode_name in _CURSES_KEYCODE_ADDINS:
+    _LASTVAL += 1
+    globals()['KEY_' + keycode_name] = _LASTVAL
 
 #: In a perfect world, terminal emulators would always send exactly what
 #: the terminfo(5) capability database plans for them, accordingly by the
@@ -370,14 +366,18 @@ DEFAULT_SEQUENCE_MIXIN = (
     (six.unichr(10), curses.KEY_ENTER),
     (six.unichr(13), curses.KEY_ENTER),
     (six.unichr(8), curses.KEY_BACKSPACE),
-    (six.unichr(9), curses.KEY_TAB),
+    (six.unichr(9), KEY_TAB),  # noqa  # pylint: disable=undefined-variable
     (six.unichr(27), curses.KEY_EXIT),
-    (six.unichr(127), curses.KEY_DC),
+    (six.unichr(127), curses.KEY_BACKSPACE),
 
     (u"\x1b[A", curses.KEY_UP),
     (u"\x1b[B", curses.KEY_DOWN),
     (u"\x1b[C", curses.KEY_RIGHT),
     (u"\x1b[D", curses.KEY_LEFT),
+    (u"\x1b[1;2A", curses.KEY_SR),
+    (u"\x1b[1;2B", curses.KEY_SF),
+    (u"\x1b[1;2C", curses.KEY_SRIGHT),
+    (u"\x1b[1;2D", curses.KEY_SLEFT),
     (u"\x1b[F", curses.KEY_END),
     (u"\x1b[H", curses.KEY_HOME),
     # not sure where these are from .. please report
@@ -390,24 +390,24 @@ DEFAULT_SEQUENCE_MIXIN = (
     # http://fossies.org/linux/rxvt/doc/rxvtRef.html#KeyCodes
     #
     # keypad, numlock on
-    (u"\x1bOM", curses.KEY_ENTER),         # return
-    (u"\x1bOj", curses.KEY_KP_MULTIPLY),   # *
-    (u"\x1bOk", curses.KEY_KP_ADD),        # +
-    (u"\x1bOl", curses.KEY_KP_SEPARATOR),  # ,
-    (u"\x1bOm", curses.KEY_KP_SUBTRACT),   # -
-    (u"\x1bOn", curses.KEY_KP_DECIMAL),    # .
-    (u"\x1bOo", curses.KEY_KP_DIVIDE),     # /
-    (u"\x1bOX", curses.KEY_KP_EQUAL),      # =
-    (u"\x1bOp", curses.KEY_KP_0),          # 0
-    (u"\x1bOq", curses.KEY_KP_1),          # 1
-    (u"\x1bOr", curses.KEY_KP_2),          # 2
-    (u"\x1bOs", curses.KEY_KP_3),          # 3
-    (u"\x1bOt", curses.KEY_KP_4),          # 4
-    (u"\x1bOu", curses.KEY_KP_5),          # 5
-    (u"\x1bOv", curses.KEY_KP_6),          # 6
-    (u"\x1bOw", curses.KEY_KP_7),          # 7
-    (u"\x1bOx", curses.KEY_KP_8),          # 8
-    (u"\x1bOy", curses.KEY_KP_9),          # 9
+    (u"\x1bOM", curses.KEY_ENTER),  # noqa return
+    (u"\x1bOj", KEY_KP_MULTIPLY),   # noqa *  # pylint: disable=undefined-variable
+    (u"\x1bOk", KEY_KP_ADD),        # noqa +  # pylint: disable=undefined-variable
+    (u"\x1bOl", KEY_KP_SEPARATOR),  # noqa ,  # pylint: disable=undefined-variable
+    (u"\x1bOm", KEY_KP_SUBTRACT),   # noqa -  # pylint: disable=undefined-variable
+    (u"\x1bOn", KEY_KP_DECIMAL),    # noqa .  # pylint: disable=undefined-variable
+    (u"\x1bOo", KEY_KP_DIVIDE),     # noqa /  # pylint: disable=undefined-variable
+    (u"\x1bOX", KEY_KP_EQUAL),      # noqa =  # pylint: disable=undefined-variable
+    (u"\x1bOp", KEY_KP_0),          # noqa 0  # pylint: disable=undefined-variable
+    (u"\x1bOq", KEY_KP_1),          # noqa 1  # pylint: disable=undefined-variable
+    (u"\x1bOr", KEY_KP_2),          # noqa 2  # pylint: disable=undefined-variable
+    (u"\x1bOs", KEY_KP_3),          # noqa 3  # pylint: disable=undefined-variable
+    (u"\x1bOt", KEY_KP_4),          # noqa 4  # pylint: disable=undefined-variable
+    (u"\x1bOu", KEY_KP_5),          # noqa 5  # pylint: disable=undefined-variable
+    (u"\x1bOv", KEY_KP_6),          # noqa 6  # pylint: disable=undefined-variable
+    (u"\x1bOw", KEY_KP_7),          # noqa 7  # pylint: disable=undefined-variable
+    (u"\x1bOx", KEY_KP_8),          # noqa 8  # pylint: disable=undefined-variable
+    (u"\x1bOy", KEY_KP_9),          # noqa 9  # pylint: disable=undefined-variable
 
     # keypad, numlock off
     (u"\x1b[1~", curses.KEY_FIND),         # find
